/* eslint-disable import/no-unresolved */
import { Plugin } from 'ckeditor5/src/core';
import { toWidget, Widget } from 'ckeditor5/src/widget';
import Footnotescommand from './footnotescommand';

/**
 * Footnotes editing functionality.
 */
export default class Footnotesediting extends Plugin {
  /**
   * @inheritdoc
   */
  static get requires() {
    return [Widget];
  }

  /**
   * @inheritdoc
   */
  init() {
    this.attrs = {
      footnotesText: 'data-text',
      footnotesValue: 'data-value',
    };
    const options = this.editor.config.get('footnotes');
    if (!options) {
      return;
    }
    const { previewURL, themeError } = options;
    this.previewUrl = previewURL;
    this.themeError =
      themeError ||
      `
        <p>${Drupal.t(
          'An error occurred while trying to preview the embedded content. Please save your work and reload this page.',
        )}<p>
        `;

    this._defineSchema();
    this._defineConverters();

    this.editor.commands.add('footnotes', new Footnotescommand(this.editor));
  }

  /**
   * Fetches the preview for the given model element.
   *
   * @param {Element} modelElement - The CKEditor model element representing footnotes.
   */
  async _fetchPreview(modelElement) {
    const query = {
      text: modelElement.getAttribute('footnotesText'),
      value: modelElement.getAttribute('footnotesValue'),
    };
    const response = await fetch(
      `${this.previewUrl}?${new URLSearchParams(query)}`,
    );
    if (response.ok) {
      return response.text();
    }

    return this.themeError;
  }

  /**
   * Registers footnotes as a block element in the DOM converter.
   */
  _defineSchema() {
    const { schema } = this.editor.model;
    schema.register('footnotes', {
      allowWhere: '$inlineObject',
      blockObject: false,
      isObject: true,
      isContent: true,
      isBlock: false,
      isInline: true,
      inlineObject: true,
      allowAttributes: Object.keys(this.attrs),
    });
    this.editor.editing.view.domConverter.blockElements.push('footnotes');
  }

  /**
   * Defines handling of drupal media element in the content lifecycle.
   *
   * @private
   */
  _defineConverters() {
    const { conversion } = this.editor;

    conversion.for('upcast').elementToElement({
      view: {
        name: 'footnotes',
      },
      model: 'footnotes',
    });

    conversion.for('dataDowncast').elementToElement({
      model: 'footnotes',
      view: {
        name: 'footnotes',
      },
    });
    conversion
      .for('editingDowncast')
      .elementToElement({
        model: 'footnotes',
        view: (modelElement, { writer }) => {
          const container = writer.createContainerElement('span');
          return toWidget(container, writer, {
            label: Drupal.t('Footnotes'),
          });
        },
      })
      .add((dispatcher) => {
        const converter = (event, data, conversionApi) => {
          const viewWriter = conversionApi.writer;
          const modelElement = data.item;
          const container = conversionApi.mapper.toViewElement(data.item);
          const footnotes = viewWriter.createRawElement('span', {
            'data-footnotes-preview': 'loading',
            class: 'footnotes-preview',
          });
          viewWriter.insert(
            viewWriter.createPositionAt(container, 0),
            footnotes,
          );
          this._fetchPreview(modelElement).then((preview) => {
            if (!footnotes) {
              return;
            }
            this.editor.editing.view.change((writer) => {
              const footnotesPreview = writer.createRawElement(
                'span',
                {
                  class: 'footnotes-preview',
                  'data-footnotes-preview': 'ready',
                },
                // eslint-disable-next-line max-nested-callbacks
                (domElement) => {
                  domElement.innerHTML = preview;
                },
              );
              writer.insert(
                writer.createPositionBefore(footnotes),
                footnotesPreview,
              );
              writer.remove(footnotes);
            });
          });
        };
        dispatcher.on('attribute:footnotesValue:footnotes', converter);
        return dispatcher;
      });

    Object.keys(this.attrs).forEach((modelKey) => {
      const attributeMapping = {
        model: {
          key: modelKey,
          name: 'footnotes',
        },
        view: {
          name: 'footnotes',
          key: this.attrs[modelKey],
        },
      };
      conversion.for('dataDowncast').attributeToAttribute(attributeMapping);
      conversion.for('upcast').attributeToAttribute(attributeMapping);
    });
  }

  /**
   * @inheritdoc
   */
  static get pluginName() {
    return 'footnotesEditing';
  }
}
